#define MOZ_SOURCE_STAMP 78ae21055d9f303be257abe155ba9dee466c0651
#define MOZ_SOURCE_REPO https://hg.mozilla.org/releases/mozilla-beta
#define MOZ_SOURCE_URL https://hg.mozilla.org/releases/mozilla-beta/rev/78ae21055d9f303be257abe155ba9dee466c0651
